using SecureShopContracts.BindingModels;
using SecureShopContracts.BusinessLogicsContracts;
using SecureShopContracts.SearchModels;
using SecureShopContracts.StoragesContracts;
using SecureShopContracts.ViewModels;
using SecureShopBusinessLogic.MailWorker;
using SecureShopDataModels.Enums;
using Microsoft.Extensions.Logging;

namespace SecureShopBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly AbstractMailWorker _abstractMailWorker;
        private readonly IClientStorage _clientStorage;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker abstractMailWorker, IClientStorage clientStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _abstractMailWorker = abstractMailWorker;
            _clientStorage = clientStorage;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderId:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.SecureId < 0)
            {
                throw new ArgumentNullException("Нет системы безопасности с таким ид", nameof(model.SecureId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество систем безопасности в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderId: {Id}. SecureId: {SecureId}. Count: {Count} Sum: {Sum}.", model.Id, model.SecureId, model.Count, model.Sum);
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            model.Status = OrderStatus.Принят;
            var order = _orderStorage.Insert(model);
            if (order == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            var client = _clientStorage.GetElement(new() { Id = order.ClientId });
            if (client == null)
            {
                _logger.LogWarning("Client not found");
                return false;
            }
            SendMail(client.Email, $"Новый заказ создан. Номер заказа - {order.Id}", $"Заказ №{order.Id} от {order.DateCreate} на сумму {order.Sum:C2} принят.");
            return true;
        }

        public bool ChangeStatus(OrderBindingModel model, OrderStatus newStatus)
        {
            var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (viewModel == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (viewModel.Status + 1 != newStatus)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }
            if (viewModel.ImplementerId.HasValue)
            {
                model.ImplementerId = viewModel.ImplementerId;
            }
            model.Status = newStatus;
            model.SecureId = viewModel.SecureId;
            model.Count = viewModel.Count;
            model.Sum = viewModel.Sum;
            model.DateCreate = viewModel.DateCreate;
            if (model.Status == OrderStatus.Готов)
            {
                model.DateImplement = DateTime.Now;
            }
            else
            {
                model.DateImplement = viewModel.DateImplement;
            }
            CheckModel(model);
            var order = _orderStorage.Update(model);
            if (order == null)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }
            var client = _clientStorage.GetElement(new() { Id = order.ClientId });
            if (client == null)
            {
                _logger.LogWarning("Client not found");
                return false;
            }
            SendMail(client.Email, $"Заказ №{order.Id}", $"Заказ №{order.Id} изменил статус на {order.Status}.");
            return true;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public void SendMail(string email, string title, string body)
        {
            _abstractMailWorker.MailSendAsync(new()
            {
                MailAddress = email,
                Subject = title,
                Text = body
            });
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выполняется);
        }

        public bool IssuedOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }

        public bool OrderReady(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }
    }
}