using SecureShopContracts.BindingModels;
using SecureShopContracts.SearchModels;
using SecureShopContracts.ViewModels;

namespace SecureShopContracts.BusinessLogicsContracts
{
    public interface IOrderLogic
    {
        List<OrderViewModel>? ReadList(OrderSearchModel? model);
        bool CreateOrder(OrderBindingModel model);
        bool TakeOrderInWork(OrderBindingModel model);
        bool OrderReady(OrderBindingModel model);
        bool IssuedOrder(OrderBindingModel model);
        OrderViewModel? ReadElement(OrderSearchModel orderSearchModel);
    }
}